#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/04/30
#Last Modify: 2018/04/30
# deeper learning insights machine learning page 600

# In the following example, we create a function that builds a gradient
# boosting classifier and graphs its cumulative loss versus the number of
# iterations. The GradientBoostingClassifier class has an oob_improvement_
# attribute and is used here calculate an estimate of the test loss on each
# iteration. This gives us a reduction in the loss compared to the previous
# iteration. This can be a very useful heuristic for determining the number of
# optimum iterations. Here, we plot the cumulative improvement of two gradient
# boosting classifiers. Each classifier is identical but for a different
# learning rate, .01 in the case of the dotted line and .001 for the solid
# line.
# 
# The learning rate shrinks the contribution of each tree, and this means that
# there is a tradeoff with the number of estimators. Here, we actually see that
# with a larger learning rate, the model appears to reach its optimum
# performance faster than the model with a lower learning rate. However, this
# models appears to achieve better results overall. What usually occurs in
# practice is that oob_improvement deviates in a pessimistic way over a large
# number of iterations.

import numpy as np
import matplotlib.pyplot as plt
from sklearn import ensemble
from sklearn.cross_validation import train_test_split
from sklearn import datasets

def gbt(params, X,y,ls):
  clf = ensemble.GradientBoostingClassifier(**params)
  clf.fit(X_train, y_train)
  cumsum = np.cumsum(clf.oob_improvement_)
  n = np.arange(params['n_estimators'])
  oob_best_iter = n[np.argmax(cumsum)]
  plt.xlabel('Iterations')
  plt.ylabel('Improvement')
  plt.axvline(x=oob_best_iter,linestyle=ls)
  plt.plot(n, cumsum, linestyle=ls)

X,y=datasets.make_blobs(n_samples=50,
                        centers=5,
                        random_state=0,
                        cluster_std=5)
X_train, X_test, y_train, y_test = train_test_split(X, y,
                                                    test_size=0.5,
                                                    random_state=9)
p1 = {'n_estimators': 1200, 
      'max_depth': 3,
      'subsample': 0.5,
      'learning_rate': 0.01,
      'min_samples_leaf': 1,
      'random_state':3}
p2 = {'n_estimators': 1200,
      'max_depth': 3,
      'subsample': 0.5,
      'learning_rate': 0.001,
      'min_samples_leaf': 1,
      'random_state':3}

gbt(p1, X,y, ls='--')
gbt(p2, X,y, ls='-')
plt.show()
